@@ -161,7 +161,7 @@ async def test_auto_header_user_agent(aiohttp_client):
     client = await aiohttp_client(app)
 
     resp = await client.get('/')
-    assert 200, resp.status
+    assert 200 == resp.status
 
 
 async def test_skip_auto_headers_user_agent(aiohttp_client):
@@ -495,6 +495,26 @@ async def test_raw_headers(aiohttp_client):
     resp.close()
 
 
+async def test_empty_header_values(aiohttp_client):
+    async def handler(request):
+        resp = web.Response()
+        resp.headers['X-Empty'] = ''
+        return resp
+
+    app = web.Application()
+    app.router.add_route('GET', '/', handler)
+    client = await aiohttp_client(app)
+    resp = await client.get('/')
+    assert resp.status == 200
+    raw_headers = tuple((bytes(h), bytes(v)) for h, v in resp.raw_headers)
+    assert raw_headers == ((b'X-Empty', b''),
+                           (b'Content-Length', b'0'),
+                           (b'Content-Type', b'application/octet-stream'),
+                           (b'Date', mock.ANY),
+                           (b'Server', mock.ANY))
+    resp.close()
+
+
 async def test_204_with_gzipped_content_encoding(aiohttp_client):
     async def handler(request):
         resp = web.StreamResponse(status=204)
@@ -2032,6 +2052,33 @@ async def test_raise_for_status(aiohttp_client):
         await client.get('/')
 
 
+async def test_raise_for_status_per_request(aiohttp_client):
+
+    async def handler_redirect(request):
+        raise web.HTTPBadRequest()
+
+    app = web.Application()
+    app.router.add_route('GET', '/', handler_redirect)
+    client = await aiohttp_client(app)
+
+    with pytest.raises(aiohttp.ClientResponseError):
+        await client.get('/', raise_for_status=True)
+
+
+async def test_raise_for_status_disable_per_request(aiohttp_client):
+
+    async def handler_redirect(request):
+        raise web.HTTPBadRequest()
+
+    app = web.Application()
+    app.router.add_route('GET', '/', handler_redirect)
+    client = await aiohttp_client(app, raise_for_status=True)
+
+    resp = await client.get('/', raise_for_status=False)
+    assert 400 == resp.status
+    resp.close()
+
+
 async def test_invalid_idna():
     session = aiohttp.ClientSession()
     try:
