@@ -12,7 +12,7 @@ from multidict import CIMultiDict, MultiDict
 from yarl import URL
 
 import aiohttp
-from aiohttp import hdrs, web
+from aiohttp import client, hdrs, web
 from aiohttp.client import ClientSession
 from aiohttp.client_reqrep import ClientRequest
 from aiohttp.connector import BaseConnector, TCPConnector
@@ -646,3 +646,22 @@ def test_client_session_custom_attr(loop):
     session = ClientSession(loop=loop)
     with pytest.warns(DeprecationWarning):
         session.custom = None
+
+
+def test_client_session_timeout_args(loop):
+    session1 = ClientSession(loop=loop)
+    assert session1._timeout == client.DEFAULT_TIMEOUT
+
+    session2 = ClientSession(loop=loop, read_timeout=20*60, conn_timeout=30*60)
+    assert session2._timeout == client.ClientTimeout(total=20*60,
+                                                     connect=30*60)
+
+    with pytest.raises(ValueError):
+        ClientSession(loop=loop,
+                      timeout=client.ClientTimeout(total=10*60),
+                      read_timeout=20*60)
+
+    with pytest.raises(ValueError):
+        ClientSession(loop=loop,
+                      timeout=client.ClientTimeout(total=10 * 60),
+                      conn_timeout=30 * 60)
