@@ -7,9 +7,11 @@ from unittest import mock
 import pytest
 
 import aiohttp
-from aiohttp import client, hdrs, helpers
+from aiohttp import client, hdrs
 from aiohttp.http import WS_KEY
 from aiohttp.log import ws_logger
+from aiohttp.streams import EofStream
+from aiohttp.test_utils import make_mocked_coro
 
 
 @pytest.fixture
@@ -27,8 +29,7 @@ def ws_key(key):
     return base64.b64encode(hashlib.sha1(key + WS_KEY).digest()).decode()
 
 
-@asyncio.coroutine
-def test_ws_connect(ws_key, loop, key_data):
+async def test_ws_connect(ws_key, loop, key_data):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -40,10 +41,10 @@ def test_ws_connect(ws_key, loop, key_data):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
-            res = yield from aiohttp.ClientSession(loop=loop).ws_connect(
+            res = await aiohttp.ClientSession(loop=loop).ws_connect(
                 'http://test.org',
                 protocols=('t1', 't2', 'chat'))
 
@@ -52,27 +53,25 @@ def test_ws_connect(ws_key, loop, key_data):
     assert hdrs.ORIGIN not in m_req.call_args[1]["headers"]
 
 
-@asyncio.coroutine
-def test_ws_connect_with_origin(key_data, loop):
+async def test_ws_connect_with_origin(key_data, loop):
     resp = mock.Mock()
     resp.status = 403
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
             origin = 'https://example.org/page.html'
             with pytest.raises(client.WSServerHandshakeError):
-                yield from aiohttp.ClientSession(loop=loop).ws_connect(
+                await aiohttp.ClientSession(loop=loop).ws_connect(
                     'http://test.org', origin=origin)
 
     assert hdrs.ORIGIN in m_req.call_args[1]["headers"]
     assert m_req.call_args[1]["headers"][hdrs.ORIGIN] == origin
 
 
-@asyncio.coroutine
-def test_ws_connect_custom_response(loop, ws_key, key_data):
+async def test_ws_connect_custom_response(loop, ws_key, key_data):
 
     class CustomResponse(client.ClientWebSocketResponse):
         def read(self, decode=False):
@@ -88,18 +87,17 @@ def test_ws_connect_custom_response(loop, ws_key, key_data):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
-            res = yield from aiohttp.ClientSession(
+            res = await aiohttp.ClientSession(
                 ws_response_class=CustomResponse, loop=loop).ws_connect(
                     'http://test.org')
 
     assert res.read() == 'customized!'
 
 
-@asyncio.coroutine
-def test_ws_connect_err_status(loop, ws_key, key_data):
+async def test_ws_connect_err_status(loop, ws_key, key_data):
     resp = mock.Mock()
     resp.status = 500
     resp.headers = {
@@ -110,19 +108,18 @@ def test_ws_connect_err_status(loop, ws_key, key_data):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
             with pytest.raises(client.WSServerHandshakeError) as ctx:
-                yield from aiohttp.ClientSession(loop=loop).ws_connect(
+                await aiohttp.ClientSession(loop=loop).ws_connect(
                     'http://test.org',
                     protocols=('t1', 't2', 'chat'))
 
     assert ctx.value.message == 'Invalid response status'
 
 
-@asyncio.coroutine
-def test_ws_connect_err_upgrade(loop, ws_key, key_data):
+async def test_ws_connect_err_upgrade(loop, ws_key, key_data):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -133,19 +130,18 @@ def test_ws_connect_err_upgrade(loop, ws_key, key_data):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
             with pytest.raises(client.WSServerHandshakeError) as ctx:
-                yield from aiohttp.ClientSession(loop=loop).ws_connect(
+                await aiohttp.ClientSession(loop=loop).ws_connect(
                     'http://test.org',
                     protocols=('t1', 't2', 'chat'))
 
     assert ctx.value.message == 'Invalid upgrade header'
 
 
-@asyncio.coroutine
-def test_ws_connect_err_conn(loop, ws_key, key_data):
+async def test_ws_connect_err_conn(loop, ws_key, key_data):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -156,19 +152,18 @@ def test_ws_connect_err_conn(loop, ws_key, key_data):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
             with pytest.raises(client.WSServerHandshakeError) as ctx:
-                yield from aiohttp.ClientSession(loop=loop).ws_connect(
+                await aiohttp.ClientSession(loop=loop).ws_connect(
                     'http://test.org',
                     protocols=('t1', 't2', 'chat'))
 
     assert ctx.value.message == 'Invalid connection header'
 
 
-@asyncio.coroutine
-def test_ws_connect_err_challenge(loop, ws_key, key_data):
+async def test_ws_connect_err_challenge(loop, ws_key, key_data):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -179,19 +174,18 @@ def test_ws_connect_err_challenge(loop, ws_key, key_data):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
             with pytest.raises(client.WSServerHandshakeError) as ctx:
-                yield from aiohttp.ClientSession(loop=loop).ws_connect(
+                await aiohttp.ClientSession(loop=loop).ws_connect(
                     'http://test.org',
                     protocols=('t1', 't2', 'chat'))
 
     assert ctx.value.message == 'Invalid challenge response'
 
 
-@asyncio.coroutine
-def test_ws_connect_common_headers(ws_key, loop, key_data):
+async def test_ws_connect_common_headers(ws_key, loop, key_data):
     """Emulate a headers dict being reused for a second ws_connect.
 
     In this scenario, we need to ensure that the newly generated secret key
@@ -199,10 +193,9 @@ def test_ws_connect_common_headers(ws_key, loop, key_data):
     """
     headers = {}
 
-    @asyncio.coroutine
-    def test_connection():
-        @asyncio.coroutine
-        def mock_get(*args, **kwargs):
+    async def test_connection():
+
+        async def mock_get(*args, **kwargs):
             resp = mock.Mock()
             resp.status = 101
             key = kwargs.get('headers').get(hdrs.SEC_WEBSOCKET_KEY)
@@ -221,7 +214,7 @@ def test_ws_connect_common_headers(ws_key, loop, key_data):
                             side_effect=mock_get) as m_req:
                 m_os.urandom.return_value = key_data
 
-                res = yield from aiohttp.ClientSession(loop=loop).ws_connect(
+                res = await aiohttp.ClientSession(loop=loop).ws_connect(
                     'http://test.org',
                     protocols=('t1', 't2', 'chat'),
                     headers=headers)
@@ -230,14 +223,13 @@ def test_ws_connect_common_headers(ws_key, loop, key_data):
         assert res.protocol == 'chat'
         assert hdrs.ORIGIN not in m_req.call_args[1]["headers"]
 
-    yield from test_connection()
+    await test_connection()
     # Generate a new ws key
     key_data = os.urandom(16)
-    yield from test_connection()
+    await test_connection()
 
 
-@asyncio.coroutine
-def test_close(loop, ws_key, key_data):
+async def test_close(loop, ws_key, key_data):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -249,34 +241,35 @@ def test_close(loop, ws_key, key_data):
         with mock.patch('aiohttp.client.os') as m_os:
             with mock.patch('aiohttp.client.ClientSession.get') as m_req:
                 m_os.urandom.return_value = key_data
-                m_req.return_value = helpers.create_future(loop)
+                m_req.return_value = loop.create_future()
                 m_req.return_value.set_result(resp)
-                writer = WebSocketWriter.return_value = mock.Mock()
+                writer = mock.Mock()
+                WebSocketWriter.return_value = writer
+                writer.close = make_mocked_coro()
 
                 session = aiohttp.ClientSession(loop=loop)
-                resp = yield from session.ws_connect(
+                resp = await session.ws_connect(
                     'http://test.org')
                 assert not resp.closed
 
                 resp._reader.feed_data(
                     aiohttp.WSMessage(aiohttp.WSMsgType.CLOSE, b'', b''), 0)
 
-                res = yield from resp.close()
+                res = await resp.close()
                 writer.close.assert_called_with(1000, b'')
                 assert resp.closed
                 assert res
                 assert resp.exception() is None
 
                 # idempotent
-                res = yield from resp.close()
+                res = await resp.close()
                 assert not res
                 assert writer.close.call_count == 1
 
-                session.close()
+                await session.close()
 
 
-@asyncio.coroutine
-def test_close_exc(loop, ws_key, key_data):
+async def test_close_eofstream(loop, ws_key, key_data):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -288,26 +281,57 @@ def test_close_exc(loop, ws_key, key_data):
         with mock.patch('aiohttp.client.os') as m_os:
             with mock.patch('aiohttp.client.ClientSession.get') as m_req:
                 m_os.urandom.return_value = key_data
-                m_req.return_value = helpers.create_future(loop)
+                m_req.return_value = loop.create_future()
                 m_req.return_value.set_result(resp)
-                WebSocketWriter.return_value = mock.Mock()
+                writer = WebSocketWriter.return_value = mock.Mock()
 
                 session = aiohttp.ClientSession(loop=loop)
-                resp = yield from session.ws_connect('http://test.org')
+                resp = await session.ws_connect('http://test.org')
+                assert not resp.closed
+
+                exc = EofStream()
+                resp._reader.set_exception(exc)
+
+                await resp.receive()
+                writer.close.assert_called_with(1000, b'')
+                assert resp.closed
+
+                await session.close()
+
+
+async def test_close_exc(loop, ws_key, key_data):
+    resp = mock.Mock()
+    resp.status = 101
+    resp.headers = {
+        hdrs.UPGRADE: hdrs.WEBSOCKET,
+        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
+    }
+    with mock.patch('aiohttp.client.WebSocketWriter') as WebSocketWriter:
+        with mock.patch('aiohttp.client.os') as m_os:
+            with mock.patch('aiohttp.client.ClientSession.get') as m_req:
+                m_os.urandom.return_value = key_data
+                m_req.return_value = loop.create_future()
+                m_req.return_value.set_result(resp)
+                writer = mock.Mock()
+                WebSocketWriter.return_value = writer
+                writer.close = make_mocked_coro()
+
+                session = aiohttp.ClientSession(loop=loop)
+                resp = await session.ws_connect('http://test.org')
                 assert not resp.closed
 
                 exc = ValueError()
                 resp._reader.set_exception(exc)
 
-                yield from resp.close()
+                await resp.close()
                 assert resp.closed
                 assert resp.exception() is exc
 
-                session.close()
+                await session.close()
 
 
-@asyncio.coroutine
-def test_close_exc2(loop, ws_key, key_data):
+async def test_close_exc2(loop, ws_key, key_data):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -319,29 +343,28 @@ def test_close_exc2(loop, ws_key, key_data):
         with mock.patch('aiohttp.client.os') as m_os:
             with mock.patch('aiohttp.client.ClientSession.get') as m_req:
                 m_os.urandom.return_value = key_data
-                m_req.return_value = helpers.create_future(loop)
+                m_req.return_value = loop.create_future()
                 m_req.return_value.set_result(resp)
                 writer = WebSocketWriter.return_value = mock.Mock()
 
-                resp = yield from aiohttp.ClientSession(loop=loop).ws_connect(
+                resp = await aiohttp.ClientSession(loop=loop).ws_connect(
                     'http://test.org')
                 assert not resp.closed
 
                 exc = ValueError()
                 writer.close.side_effect = exc
 
-                yield from resp.close()
+                await resp.close()
                 assert resp.closed
                 assert resp.exception() is exc
 
                 resp._closed = False
                 writer.close.side_effect = asyncio.CancelledError()
                 with pytest.raises(asyncio.CancelledError):
-                    yield from resp.close()
+                    await resp.close()
 
 
-@asyncio.coroutine
-def test_send_data_after_close(ws_key, key_data, loop, mocker):
+async def test_send_data_after_close(ws_key, key_data, loop, mocker):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -352,10 +375,10 @@ def test_send_data_after_close(ws_key, key_data, loop, mocker):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
-            resp = yield from aiohttp.ClientSession(loop=loop).ws_connect(
+            resp = await aiohttp.ClientSession(loop=loop).ws_connect(
                 'http://test.org')
             resp._writer._closing = True
 
@@ -366,13 +389,12 @@ def test_send_data_after_close(ws_key, key_data, loop, mocker):
                                (resp.send_str, ('s',)),
                                (resp.send_bytes, (b'b',)),
                                (resp.send_json, ({},))):
-                meth(*args)
+                await meth(*args)
                 assert ws_logger.warning.called
                 ws_logger.warning.reset_mock()
 
 
-@asyncio.coroutine
-def test_send_data_type_errors(ws_key, key_data, loop):
+async def test_send_data_type_errors(ws_key, key_data, loop):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -384,20 +406,22 @@ def test_send_data_type_errors(ws_key, key_data, loop):
         with mock.patch('aiohttp.client.os') as m_os:
             with mock.patch('aiohttp.client.ClientSession.get') as m_req:
                 m_os.urandom.return_value = key_data
-                m_req.return_value = helpers.create_future(loop)
+                m_req.return_value = loop.create_future()
                 m_req.return_value.set_result(resp)
                 WebSocketWriter.return_value = mock.Mock()
 
-                resp = yield from aiohttp.ClientSession(loop=loop).ws_connect(
+                resp = await aiohttp.ClientSession(loop=loop).ws_connect(
                     'http://test.org')
 
-                pytest.raises(TypeError, resp.send_str, b's')
-                pytest.raises(TypeError, resp.send_bytes, 'b')
-                pytest.raises(TypeError, resp.send_json, set())
+                with pytest.raises(TypeError):
+                    await resp.send_str(b's')
+                with pytest.raises(TypeError):
+                    await resp.send_bytes('b')
+                with pytest.raises(TypeError):
+                    await resp.send_json(set())
 
 
-@asyncio.coroutine
-def test_reader_read_exception(ws_key, key_data, loop):
+async def test_reader_read_exception(ws_key, key_data, loop):
     hresp = mock.Mock()
     hresp.status = 101
     hresp.headers = {
@@ -409,37 +433,37 @@ def test_reader_read_exception(ws_key, key_data, loop):
         with mock.patch('aiohttp.client.os') as m_os:
             with mock.patch('aiohttp.client.ClientSession.get') as m_req:
                 m_os.urandom.return_value = key_data
-                m_req.return_value = helpers.create_future(loop)
+                m_req.return_value = loop.create_future()
                 m_req.return_value.set_result(hresp)
-                WebSocketWriter.return_value = mock.Mock()
+
+                writer = mock.Mock()
+                WebSocketWriter.return_value = writer
+                writer.close = make_mocked_coro()
 
                 session = aiohttp.ClientSession(loop=loop)
-                resp = yield from session.ws_connect('http://test.org')
+                resp = await session.ws_connect('http://test.org')
 
                 exc = ValueError()
                 resp._reader.set_exception(exc)
 
-                msg = yield from resp.receive()
+                msg = await resp.receive()
                 assert msg.type == aiohttp.WSMsgType.ERROR
-                assert msg.type is msg.tp
                 assert resp.exception() is exc
 
-                session.close()
+                await session.close()
 
 
-@asyncio.coroutine
-def test_receive_runtime_err(loop):
+async def test_receive_runtime_err(loop):
     resp = client.ClientWebSocketResponse(
         mock.Mock(), mock.Mock(), mock.Mock(), mock.Mock(), 10.0,
         True, True, loop)
     resp._waiting = True
 
     with pytest.raises(RuntimeError):
-        yield from resp.receive()
+        await resp.receive()
 
 
-@asyncio.coroutine
-def test_ws_connect_close_resp_on_err(loop, ws_key, key_data):
+async def test_ws_connect_close_resp_on_err(loop, ws_key, key_data):
     resp = mock.Mock()
     resp.status = 500
     resp.headers = {
@@ -450,18 +474,17 @@ def test_ws_connect_close_resp_on_err(loop, ws_key, key_data):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
             with pytest.raises(client.WSServerHandshakeError):
-                yield from aiohttp.ClientSession(loop=loop).ws_connect(
+                await aiohttp.ClientSession(loop=loop).ws_connect(
                     'http://test.org',
                     protocols=('t1', 't2', 'chat'))
             resp.close.assert_called_with()
 
 
-@asyncio.coroutine
-def test_ws_connect_non_overlapped_protocols(ws_key, loop, key_data):
+async def test_ws_connect_non_overlapped_protocols(ws_key, loop, key_data):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -473,18 +496,17 @@ def test_ws_connect_non_overlapped_protocols(ws_key, loop, key_data):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
-            res = yield from aiohttp.ClientSession(loop=loop).ws_connect(
+            res = await aiohttp.ClientSession(loop=loop).ws_connect(
                 'http://test.org',
                 protocols=('t1', 't2', 'chat'))
 
     assert res.protocol is None
 
 
-@asyncio.coroutine
-def test_ws_connect_non_overlapped_protocols_2(ws_key, loop, key_data):
+async def test_ws_connect_non_overlapped_protocols_2(ws_key, loop, key_data):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -496,11 +518,11 @@ def test_ws_connect_non_overlapped_protocols_2(ws_key, loop, key_data):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
             connector = aiohttp.TCPConnector(loop=loop, force_close=True)
-            res = yield from aiohttp.ClientSession(
+            res = await aiohttp.ClientSession(
                 connector=connector, loop=loop).ws_connect(
                 'http://test.org',
                 protocols=('t1', 't2', 'chat'))
@@ -509,8 +531,7 @@ def test_ws_connect_non_overlapped_protocols_2(ws_key, loop, key_data):
     del res
 
 
-@asyncio.coroutine
-def test_ws_connect_deflate(loop, ws_key, key_data):
+async def test_ws_connect_deflate(loop, ws_key, key_data):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -522,18 +543,50 @@ def test_ws_connect_deflate(loop, ws_key, key_data):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
-            res = yield from aiohttp.ClientSession(loop=loop).ws_connect(
+            res = await aiohttp.ClientSession(loop=loop).ws_connect(
                 'http://test.org', compress=15)
 
     assert res.compress == 15
     assert res.client_notakeover is False
 
 
-@asyncio.coroutine
-def test_ws_connect_deflate_server_not_support(loop, ws_key, key_data):
+async def test_ws_connect_deflate_per_message(loop, ws_key, key_data):
+    resp = mock.Mock()
+    resp.status = 101
+    resp.headers = {
+        hdrs.UPGRADE: hdrs.WEBSOCKET,
+        hdrs.CONNECTION: hdrs.UPGRADE,
+        hdrs.SEC_WEBSOCKET_ACCEPT: ws_key,
+        hdrs.SEC_WEBSOCKET_EXTENSIONS: 'permessage-deflate',
+    }
+    with mock.patch('aiohttp.client.WebSocketWriter') as WebSocketWriter:
+        with mock.patch('aiohttp.client.os') as m_os:
+            with mock.patch('aiohttp.client.ClientSession.get') as m_req:
+                m_os.urandom.return_value = key_data
+                m_req.return_value = loop.create_future()
+                m_req.return_value.set_result(resp)
+                writer = WebSocketWriter.return_value = mock.Mock()
+                send = writer.send = make_mocked_coro()
+
+                session = aiohttp.ClientSession(loop=loop)
+                resp = await session.ws_connect('http://test.org')
+
+                await resp.send_str('string', compress=-1)
+                send.assert_called_with('string', binary=False, compress=-1)
+
+                await resp.send_bytes(b'bytes', compress=15)
+                send.assert_called_with(b'bytes', binary=True, compress=15)
+
+                await resp.send_json([{}], compress=-9)
+                send.assert_called_with('[{}]', binary=False, compress=-9)
+
+                await session.close()
+
+
+async def test_ws_connect_deflate_server_not_support(loop, ws_key, key_data):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -544,18 +597,17 @@ def test_ws_connect_deflate_server_not_support(loop, ws_key, key_data):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
-            res = yield from aiohttp.ClientSession(loop=loop).ws_connect(
+            res = await aiohttp.ClientSession(loop=loop).ws_connect(
                 'http://test.org', compress=15)
 
     assert res.compress == 0
     assert res.client_notakeover is False
 
 
-@asyncio.coroutine
-def test_ws_connect_deflate_notakeover(loop, ws_key, key_data):
+async def test_ws_connect_deflate_notakeover(loop, ws_key, key_data):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -568,18 +620,17 @@ def test_ws_connect_deflate_notakeover(loop, ws_key, key_data):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
-            res = yield from aiohttp.ClientSession(loop=loop).ws_connect(
+            res = await aiohttp.ClientSession(loop=loop).ws_connect(
                 'http://test.org', compress=15)
 
     assert res.compress == 15
     assert res.client_notakeover is True
 
 
-@asyncio.coroutine
-def test_ws_connect_deflate_client_wbits(loop, ws_key, key_data):
+async def test_ws_connect_deflate_client_wbits(loop, ws_key, key_data):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -592,18 +643,17 @@ def test_ws_connect_deflate_client_wbits(loop, ws_key, key_data):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
-            res = yield from aiohttp.ClientSession(loop=loop).ws_connect(
+            res = await aiohttp.ClientSession(loop=loop).ws_connect(
                 'http://test.org', compress=15)
 
     assert res.compress == 10
     assert res.client_notakeover is False
 
 
-@asyncio.coroutine
-def test_ws_connect_deflate_client_wbits_bad(loop, ws_key, key_data):
+async def test_ws_connect_deflate_client_wbits_bad(loop, ws_key, key_data):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -616,16 +666,15 @@ def test_ws_connect_deflate_client_wbits_bad(loop, ws_key, key_data):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
             with pytest.raises(client.WSServerHandshakeError):
-                yield from aiohttp.ClientSession(loop=loop).ws_connect(
+                await aiohttp.ClientSession(loop=loop).ws_connect(
                     'http://test.org', compress=15)
 
 
-@asyncio.coroutine
-def test_ws_connect_deflate_server_ext_bad(loop, ws_key, key_data):
+async def test_ws_connect_deflate_server_ext_bad(loop, ws_key, key_data):
     resp = mock.Mock()
     resp.status = 101
     resp.headers = {
@@ -637,9 +686,9 @@ def test_ws_connect_deflate_server_ext_bad(loop, ws_key, key_data):
     with mock.patch('aiohttp.client.os') as m_os:
         with mock.patch('aiohttp.client.ClientSession.get') as m_req:
             m_os.urandom.return_value = key_data
-            m_req.return_value = helpers.create_future(loop)
+            m_req.return_value = loop.create_future()
             m_req.return_value.set_result(resp)
 
             with pytest.raises(client.WSServerHandshakeError):
-                yield from aiohttp.ClientSession(loop=loop).ws_connect(
+                await aiohttp.ClientSession(loop=loop).ws_connect(
                     'http://test.org', compress=15)
