@@ -10,27 +10,30 @@ from aiohttp.test_utils import AioHTTPTestCase, unittest_run_loop
 
 @pytest.mark.skipif(platform.system() == "Windows",
                     reason="the test is not valid for Windows")
-@asyncio.coroutine
-def test_subprocess_co(loop):
+async def test_subprocess_co(loop):
     assert isinstance(threading.current_thread(), threading._MainThread)
-    proc = yield from asyncio.create_subprocess_shell(
+    proc = await asyncio.create_subprocess_shell(
         "exit 0", loop=loop, stdin=asyncio.subprocess.DEVNULL,
         stdout=asyncio.subprocess.DEVNULL, stderr=asyncio.subprocess.DEVNULL)
-    yield from proc.wait()
+    await proc.wait()
 
 
 class TestCase(AioHTTPTestCase):
-    @asyncio.coroutine
-    def get_application(self):
+    async def get_application(self):
         app = web.Application()
         app.on_startup.append(self.on_startup_hook)
         return app
 
-    @asyncio.coroutine
-    def on_startup_hook(self, app):
+    async def on_startup_hook(self, app):
         self.startup_loop = app.loop
 
     @unittest_run_loop
-    @asyncio.coroutine
-    def test_on_startup_hook(self):
-        assert self.startup_loop is not None
+    async def test_on_startup_hook(self):
+        self.assertIsNotNone(self.startup_loop)
+
+    def test_default_loop(self):
+        self.assertIs(self.loop, asyncio.get_event_loop())
+
+
+def test_default_loop(loop):
+    assert asyncio.get_event_loop() is loop
