@@ -1,4 +1,5 @@
-import re
+import os
+import platform
 import sys
 
 import pytest
@@ -11,93 +12,73 @@ pytest_plugins = 'aiohttp.pytest_plugin'
 '''
 
 
+IS_PYPY = platform.python_implementation() == 'PyPy'
+
+
 def test_aiohttp_plugin(testdir):
     testdir.makepyfile("""\
-import asyncio
 import pytest
 from unittest import mock
 
 from aiohttp import web
 
-@asyncio.coroutine
-def hello(request):
+
+async def hello(request):
     return web.Response(body=b'Hello, world')
 
 
-def create_app(loop):
+def create_app(loop=None):
     app = web.Application()
     app.router.add_route('GET', '/', hello)
     return app
 
 
-@asyncio.coroutine
-def test_hello(test_client):
-    client = yield from test_client(create_app)
-    resp = yield from client.get('/')
+async def test_hello(aiohttp_client):
+    client = await aiohttp_client(create_app)
+    resp = await client.get('/')
     assert resp.status == 200
-    text = yield from resp.text()
+    text = await resp.text()
     assert 'Hello, world' in text
 
 
-@asyncio.coroutine
-def test_hello_from_app(test_client, loop):
+async def test_hello_from_app(aiohttp_client, loop):
     app = web.Application()
     app.router.add_get('/', hello)
-    client = yield from test_client(app)
-    resp = yield from client.get('/')
+    client = await aiohttp_client(app)
+    resp = await client.get('/')
     assert resp.status == 200
-    text = yield from resp.text()
+    text = await resp.text()
     assert 'Hello, world' in text
 
 
-@asyncio.coroutine
-def test_hello_with_loop(test_client, loop):
-    client = yield from test_client(create_app)
-    resp = yield from client.get('/')
+async def test_hello_with_loop(aiohttp_client, loop):
+    client = await aiohttp_client(create_app)
+    resp = await client.get('/')
     assert resp.status == 200
-    text = yield from resp.text()
+    text = await resp.text()
     assert 'Hello, world' in text
 
 
-@asyncio.coroutine
-def test_hello_fails(test_client):
-    client = yield from test_client(create_app)
-    resp = yield from client.get('/')
-    assert resp.status == 200
-    text = yield from resp.text()
-    assert 'Hello, wield' in text
-
-
-@asyncio.coroutine
-def test_hello_with_fake_loop(test_client):
-    with pytest.raises(RuntimeError):
-        fake_loop = mock.Mock()
-        yield from test_client(web.Application(loop=fake_loop))
-
-
-@asyncio.coroutine
-def test_set_args(test_client, loop):
+async def test_set_args(aiohttp_client, loop):
     with pytest.raises(AssertionError):
         app = web.Application()
-        yield from test_client(app, 1, 2, 3)
+        await aiohttp_client(app, 1, 2, 3)
 
 
-@asyncio.coroutine
-def test_set_keyword_args(test_client, loop):
+async def test_set_keyword_args(aiohttp_client, loop):
     app = web.Application()
     with pytest.raises(TypeError):
-        yield from test_client(app, param=1)
+        await aiohttp_client(app, param=1)
 
 
-@asyncio.coroutine
-def test_noop():
+async def test_noop():
     pass
 
 
-@asyncio.coroutine
-def previous(request):
+async def previous(request):
     if request.method == 'POST':
-        request.app['value'] = (yield from request.post())['value']
+        with pytest.warns(DeprecationWarning):
+            request.app['value'] = (await request.post())['value']
         return web.Response(body=b'thanks for the data')
     else:
         v = request.app.get('value', 'unknown')
@@ -105,35 +86,34 @@ def previous(request):
 
 
 def create_stateful_app(loop):
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('*', '/', previous)
     return app
 
 
 @pytest.fixture
-def cli(loop, test_client):
-    return loop.run_until_complete(test_client(create_stateful_app))
+def cli(loop, aiohttp_client):
+    return loop.run_until_complete(aiohttp_client(create_stateful_app))
 
 
-@asyncio.coroutine
-def test_set_value(cli):
-    resp = yield from cli.post('/', data={'value': 'foo'})
+async def test_set_value(cli):
+    resp = await cli.post('/', data={'value': 'foo'})
     assert resp.status == 200
-    text = yield from resp.text()
+    text = await resp.text()
     assert text == 'thanks for the data'
     assert cli.server.app['value'] == 'foo'
 
 
-@asyncio.coroutine
-def test_get_value(cli):
-    resp = yield from cli.get('/')
+async def test_get_value(cli):
+    resp = await cli.get('/')
     assert resp.status == 200
-    text = yield from resp.text()
+    text = await resp.text()
     assert text == 'value: unknown'
-    cli.server.app['value'] = 'bar'
-    resp = yield from cli.get('/')
+    with pytest.warns(DeprecationWarning):
+        cli.server.app['value'] = 'bar'
+    resp = await cli.get('/')
     assert resp.status == 200
-    text = yield from resp.text()
+    text = await resp.text()
     assert text == 'value: bar'
 
 
@@ -141,25 +121,39 @@ def test_noncoro():
     assert True
 
 
-@asyncio.coroutine
-def test_client_failed_to_create(test_client):
+async def test_failed_to_create_client(aiohttp_client):
 
     def make_app(loop):
         raise RuntimeError()
 
     with pytest.raises(RuntimeError):
-        yield from test_client(make_app)
+        await aiohttp_client(make_app)
+
+
+async def test_custom_port_aiohttp_client(aiohttp_client, aiohttp_unused_port):
+    port = aiohttp_unused_port()
+    client = await aiohttp_client(create_app, server_kwargs={'port': port})
+    assert client.port == port
+    resp = await client.get('/')
+    assert resp.status == 200
+    text = await resp.text()
+    assert 'Hello, world' in text
+
+
+async def test_custom_port_test_server(aiohttp_server, aiohttp_unused_port):
+    app = create_app()
+    port = aiohttp_unused_port()
+    server = await aiohttp_server(app, port=port)
+    assert server.port == port
 
 """)
     testdir.makeconftest(CONFTEST)
-    result = testdir.runpytest('-p', 'no:sugar', '--loop=pyloop')
-    result.assert_outcomes(passed=11, failed=1)
+    result = testdir.runpytest('-p', 'no:sugar', '--aiohttp-loop=pyloop')
+    result.assert_outcomes(passed=12)
 
 
-@pytest.mark.skipif(sys.version_info < (3, 5), reason='old python')
-def test_warning_checks(testdir, capsys):
+def test_warning_checks(testdir):
     testdir.makepyfile("""\
-import asyncio
 
 async def foobar():
     return 123
@@ -172,23 +166,25 @@ async def test_bad():
     foobar()
 """)
     testdir.makeconftest(CONFTEST)
-    result = testdir.runpytest('-p', 'no:sugar', '-s', '--loop=pyloop')
-    result.assert_outcomes(passed=1, failed=1)
-    stdout, _ = capsys.readouterr()
-    assert ("test_warning_checks.py:__LINE__:coroutine 'foobar' was "
-            "never awaited" in re.sub('\d{2,}', '__LINE__', stdout))
+    result = testdir.runpytest('-p', 'no:sugar', '-s', '-W',
+                               'default', '--aiohttp-loop=pyloop')
+    expected_outcomes = (
+        {'failed': 0, 'passed': 2}
+        if IS_PYPY and bool(os.environ.get('PYTHONASYNCIODEBUG'))
+        else {'failed': 1, 'passed': 1}
+    )
+    """Under PyPy "coroutine 'foobar' was never awaited" does not happen."""
+    result.assert_outcomes(**expected_outcomes)
 
 
 def test_aiohttp_plugin_async_fixture(testdir, capsys):
     testdir.makepyfile("""\
-import asyncio
 import pytest
 
 from aiohttp import web
 
 
-@asyncio.coroutine
-def hello(request):
+async def hello(request):
     return web.Response(body=b'Hello, world')
 
 
@@ -199,28 +195,24 @@ def create_app(loop):
 
 
 @pytest.fixture
-@asyncio.coroutine
-def cli(test_client):
-    client = yield from test_client(create_app)
+async def cli(aiohttp_client):
+    client = await aiohttp_client(create_app)
     return client
 
 
 @pytest.fixture
-@asyncio.coroutine
-def foo():
+async def foo():
     return 42
 
 
 @pytest.fixture
-@asyncio.coroutine
-def bar(request):
+async def bar(request):
     # request should be accessible in async fixtures if needed
     return request.function
 
 
-@asyncio.coroutine
-def test_hello(cli):
-    resp = yield from cli.get('/')
+async def test_hello(cli):
+    resp = await cli.get('/')
     assert resp.status == 200
 
 
@@ -237,7 +229,7 @@ def test_bar(loop, bar):
     assert bar is test_bar
 """)
     testdir.makeconftest(CONFTEST)
-    result = testdir.runpytest('-p', 'no:sugar', '--loop=pyloop')
+    result = testdir.runpytest('-p', 'no:sugar', '--aiohttp-loop=pyloop')
     result.assert_outcomes(passed=3, error=1)
     result.stdout.fnmatch_lines(
         "*Asynchronous fixtures must depend on the 'loop' fixture "
@@ -248,7 +240,6 @@ def test_bar(loop, bar):
 @pytest.mark.skipif(sys.version_info < (3, 6), reason='old python')
 def test_aiohttp_plugin_async_gen_fixture(testdir):
     testdir.makepyfile("""\
-import asyncio
 import pytest
 from unittest import mock
 
@@ -269,8 +260,8 @@ def create_app(loop):
 
 
 @pytest.fixture
-async def cli(test_client):
-    yield await test_client(create_app)
+async def cli(aiohttp_client):
+    yield await aiohttp_client(create_app)
     canary()
 
 
@@ -283,5 +274,5 @@ def test_finalized():
     assert canary.called is True
 """)
     testdir.makeconftest(CONFTEST)
-    result = testdir.runpytest('-p', 'no:sugar', '--loop=pyloop')
+    result = testdir.runpytest('-p', 'no:sugar', '--aiohttp-loop=pyloop')
     result.assert_outcomes(passed=2)
